@@ -43,6 +43,8 @@
 #include <common/na-xml-names.h>
 #include <common/na-xml-writer.h>
 
+#include "console-utils.h"
+
 static gchar     *label           = "";
 static gchar     *tooltip         = "";
 static gchar     *icon            = "";
@@ -80,7 +82,7 @@ static GOptionEntry entries[] = {
 static GOptionEntry output_entries[] = {
 
 	{ "output-gconf"         , 'g', 0, G_OPTION_ARG_NONE        , &output_gconf   , N_("Directly import the newly created action in GConf configuration"), NULL },
-	{ "output-dir"           , 'o', 0, G_OPTION_ARG_FILENAME    , &output_dir     , N_("The folder where to write the new action as a GConf dump output [default: stdout]"), N_("DIR") },
+	{ "output-dir"           , 'o', 0, G_OPTION_ARG_FILENAME    , &output_dir     , N_("The URI of the folder where to write the new action as a GConf dump output [default: stdout]"), N_("URI") },
 	{ NULL }
 };
 
@@ -100,6 +102,7 @@ main( int argc, char** argv )
 	gchar *help;
 
 	g_type_init();
+	console_init_log_handler();
 
 	context = init_options();
 
